from __future__ import absolute_import, division, print_function
__metaclass__ = type

import os
import sys

import ansible.module_utils.external1

from ansible.module_utils.externalpkg.extmod import path as epem_path

def main():
    pass
